/*
   unibdf2hex - program to convert a BDF file into a unifont.hex file.

   Author: Paul Hardy, January 2008

   Note: currently this has hard-coded code points for glyphs extracted
   from Wen Quan Yi to create the Unifont source file "wqy.hex".

   Copyright (C) 2008, 2013 Paul Hardy

   LICENSE:

      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXBUF 256


int
main()
{
   int i;
   int rownum;  /* number of lines we output in bitmap */
   int thispoint;
   char inbuf[MAXBUF];
   int bbxx, bbxy, bbxxoff, bbxyoff;

   int descent=2; /* font descent wrt baseline */
   int startrow;  /* row to start glyph        */
   int rowlen;    /* number of digits per row  */
   unsigned rowout;

   while (fgets (inbuf, MAXBUF - 1, stdin)) {
      if (strncmp (inbuf, "ENCODING ", 9)) continue;
      sscanf (&inbuf[9], "%d", &thispoint); /* get code point */

      while (fgets (inbuf, MAXBUF - 1, stdin)) {
         if (!strncmp (inbuf, "DWIDTH ", 7)) break;
      }
      sscanf (&inbuf[7], "%d", &rowlen);
      rowlen >>= 2;

      /* Read bounding box values from BBX line */
      while (fgets (inbuf, MAXBUF - 1, stdin)) {
        if (!strncmp (inbuf, "BBX ", 4)) break;
      }
      sscanf (&inbuf[4], "%d %d %d %d", &bbxx, &bbxy, &bbxxoff, &bbxyoff);

      /* Find BITMAP start */
      while (fgets (inbuf, MAXBUF - 1, stdin)) {
        if (!strncmp (inbuf, "BITMAP", 6)) break;
      }

      fprintf (stdout, "%04X:", thispoint);
      rownum = 0;
      /* Print initial blank rows */
      startrow = descent + bbxyoff + bbxy;

      /* Force everything to 16 pixels wide */
      for (i = 16; i > startrow; i--) {
         fprintf (stdout, "%0*d", rowlen, 0);
         rownum++;
      }
      /* Copy bitmap until END */
      while (fgets (inbuf, MAXBUF - 1, stdin)) {
         if (!strncmp (inbuf, "END", 3)) break;
         sscanf (inbuf, "%X", &rowout);
         if (rowlen >= 4 && bbxx <= 8) rowout <<= 8;  /* force 8x16 input to 16x16 grid */
         rowout >>= bbxxoff;
         fprintf (stdout, "%0*X", rowlen, rowout);
         rownum++;
      }

      /* Pad empty lines until glyph has sufficient height */
      while (rownum < 16) {
         fprintf (stdout, "%0*d", rowlen, 0);
         rownum++;
      }
      fprintf (stdout,"\n");
   }
   exit (0);
}
